#ifndef __CDX8Graphics_H__
#define __CDX8Graphics_H__
//***************************************************************************
//**
//** Alias25 (C) 2000
//**
//** File: CDX8Graphics.h
//**
//** Date: 25/11/2000
//**
//** Author: Pierre Renaux
//**
//** Desc: DirectX8 Graphics Wrapper
//**
//***************************************************************************
//===========================================================================
//    INTERFACE REQUIRED HEADERS
//===========================================================================
#include "agamelib.h"     // Comment it if you don't want to use
                          // the game library.
#include <windows.h>
#include <d3d8.h>

//===========================================================================
//    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//===========================================================================
#ifndef __AGAMELIB_H__
 #define U32 unsigned long
#endif

//===========================================================================
//    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//===========================================================================
//===========================================================================
//    INTERFACE STRUCTURES / UTILITY CLASSES
//===========================================================================
//===========================================================================
//    INTERFACE DATA DECLARATIONS
//===========================================================================
//===========================================================================
//    INTERFACE FUNCTION PROTOTYPES
//===========================================================================
//===========================================================================
//    INTERFACE OBJECT CLASS DEFINITIONS
//===========================================================================
// CDX8Graphics
class CDX8Graphics
{
// Public functions /////////////////////////////////////////////////////////
public:
 CDX8Graphics(HWND wnd = NULL);
 CDX8Graphics(HWND wnd, U32 adapter, int w, int h, D3DFORMAT pixformat = D3DFMT_UNKNOWN,
	          U32 behavior = D3DCREATE_SOFTWARE_VERTEXPROCESSING);
 CDX8Graphics(U32 adapter, int w, int h, D3DFORMAT pixformat = D3DFMT_UNKNOWN,
	          U32 behavior = D3DCREATE_SOFTWARE_VERTEXPROCESSING);
 ~CDX8Graphics();

 // Initialise
 void Initialise(HWND wnd = NULL);

 // Uninitialise
 void Uninitialise(void);

 // Return true if the object has been well initialised by the constructor
 // and Initialise.
 BOOL AllOk() { return mp_all_ok; }

 // Change the current video mode
 BOOL Gfx(U32 adapter, int w, int h, D3DFORMAT pixformat = D3DFMT_UNKNOWN,
	          U32 behavior = D3DCREATE_SOFTWARE_VERTEXPROCESSING);

 // Switch between fullscreen and windowed mode
 BOOL Switch(BOOL fullscreen = 2);

 // Return the D3DDevice8 object
 LPDIRECT3DDEVICE8 Dev() { return mp_d3d_dev; }

 // Enumeration functions
 U32  NumAdapter();
 BOOL AdapterInfo(U32 i, D3DCAPS8 *caps, char *name = NULL, char *desc = NULL);
 U32  AdapterNumMode(U32 i);
 BOOL AdapterMode(U32 i, U32 imode, int *w, int *h, D3DFORMAT *format);

 // Quick Caps test functions
 BOOL AdapterCapsWindowed(U32 i);

 // Return the informations about the current gfx mode
 U32 W() { return mp_w; }
 U32 H() { return mp_h; }
 D3DFORMAT PixelFormat() { return mp_pixformat; }
 BOOL FullScreen()   { return mp_fullscreen; }
 U32 CurrentAdapter()    { return mp_current_adapter; }

 // Virtual Member to deal with window's focus lost/gain
 virtual int SwitchIn(void)  { return 1; }
 virtual int SwitchOut(void) { return 1; }

// Private functions ////////////////////////////////////////////////////////
private:
 // Enumerate present hardware.
 BOOL mpEnumerateHardware();

 // Return the desktop display mode
 D3DFORMAT mpDesktopMode(int *w = NULL, int *h = NULL);

 // Change the style and the size of the window to use a windowed gfx mode.
 void mpWindowedStyle(int w, int h);

 // Change the style and the size of the window to use a fullscreen gfx mode.
 void mpFullScreenStyle(int w, int h);

 // Reset all variables that need to be, before a gfx mode initialisation.
 void mpCheckUpBeforeGfxInitialisation();

 // Initialise a windowed gfx mode and show the window
 BOOL mpWindowedInit(U32 adapter, int w, int h, U32 behavior);

 // Initialise a fullscreen gfx
 BOOL mpFullScreenInit(U32 adapter, int w, int h, D3DFORMAT pixformat, U32 behavior);

 // Update the info variables
 void mpUpdateInfo(U32 adapter=0, int w=0, int h=0, D3DFORMAT pixformat=D3DFMT_UNKNOWN, BOOL fullscreen=FALSE);

// Public data //////////////////////////////////////////////////////////////
public:
 
// Private Data /////////////////////////////////////////////////////////////
private:
 BOOL  mp_all_ok;   // To be sure that all is ok
 HWND      mp_wnd;      // The used window
 HINSTANCE mp_d3d8_dll; // The D3D8 DLL

 // Informations about the current gfx mode.
 U32       mp_w;
 U32       mp_h;
 D3DFORMAT mp_pixformat;
 BOOL  mp_fullscreen;

 // Current adapter index
 U32       mp_current_adapter;

 // Adapter array
 U32       mp_num_adapter;
 struct DX8GRAPHIC_ADAPTER *mp_adapters;

 // Dynamicaly loaded DX8 functions
 IDirect3D8 *(WINAPI *mpDirect3DCreate8)(UINT SDKVersion);

 // DirectX8 Objects
 LPDIRECT3D8             mp_d3d_obj; // Used to create the D3DDevice
 LPDIRECT3DDEVICE8       mp_d3d_dev; // Our rendering device
};

//===========================================================================
//    INTERFACE TRAILING HEADERS
//===========================================================================

//***************************************************************************
//**
//**    END HEADER CDX8Graphics.h
//**
//***************************************************************************
#endif // __CDX8Graphics_H__
